<?php

namespace UdeyTech\CategoryHelper\Setup\Patch\Data;

use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface;
use Magento\Catalog\Model\Category;

class AddCustomCategoryAttribute implements DataPatchInterface
{
    private $eavSetupFactory;
    private $moduleDataSetup;

    public function __construct(
        EavSetupFactory $eavSetupFactory,
        ModuleDataSetupInterface $moduleDataSetup
    ) {
        $this->eavSetupFactory = $eavSetupFactory;
        $this->moduleDataSetup = $moduleDataSetup;
    }

    public function apply()
    {
        $this->moduleDataSetup->getConnection()->startSetup();

        $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);

        $attributes = [
            'first_tab' => [
                'type' => 'text',
                'label' => 'First Tab',
                'input' => 'textarea',
                'sort_order' => 100,
            ],
            'second_tab' => [
                'type' => 'text',
                'label' => 'Second Tab',
                'input' => 'textarea',
                'sort_order' => 110,
            ],
            'third_tab' => [
                'type' => 'text',
                'label' => 'Third Tab',
                'input' => 'textarea',
                'sort_order' => 120,
            ],
        ];

        foreach ($attributes as $attributeCode => $attributeOptions) {
            $eavSetup->addAttribute(
                Category::ENTITY,
                $attributeCode,
                array_merge(
                    $attributeOptions,
                    [
                        'group' => 'Custom Category Group',
                        'required' => false,
                        'global' => ScopedAttributeInterface::SCOPE_STORE,
                        'visible' => true,
                        'user_defined' => true,
                        'backend' => '',
                        'is_wysiwyg_enabled' => true, // Optional: Enable WYSIWYG editor if needed
                        'input' => 'textarea',
                    ]
                )
            );
        }

        $this->moduleDataSetup->getConnection()->endSetup();
    }

    public static function getDependencies()
    {
        return [];
    }

    public function getAliases()
    {
        return [];
    }
}
