<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

declare(strict_types=1);

namespace Magefan\GuestToCustomer\Block\CustomerAccount;

use Magento\Customer\Model\Session;
use Magefan\GuestToCustomer\Model\Order\GuestOrderManagement;
use Magento\Customer\Model\CustomerRegistry;
use Magento\Framework\View\Element\Template\Context;
use Magefan\GuestToCustomer\Model\Config;

class AssignLink extends \Magento\Framework\View\Element\Template
{
    /**
     * @var Session
     */
    private $customerSession;

    /**
     * @var GuestOrderManagement
     */
    private $guestOrderManagement;

    /**
     * @var CustomerRegistry
     */
    private $customerRegistry;

    /**
     * @var Config
     */
    private $config;

    /**
     * @param Context $context
     * @param array $data
     * @param Session $customerSession
     * @param GuestOrderManagement $guestOrderManagement
     * @param CustomerRegistry $customerRegistry
     * @param Config $config
     */
    public function __construct(
        Context $context,
        Session $customerSession,
        GuestOrderManagement $guestOrderManagement,
        CustomerRegistry $customerRegistry,
        Config $config,
        array $data = []
    ) {
        $this->customerSession = $customerSession;
        $this->guestOrderManagement = $guestOrderManagement;
        $this->customerRegistry = $customerRegistry;
        $this->config = $config;
        parent::__construct($context, $data);
    }

    /**
     * Return true if exists orders with the same email as in customer that was placed under the guest account
     * @return bool
     */
    public function isCustomerHasUnassignedGuestOrders(): bool
    {
        return $this->guestOrderManagement->isCustomerHasUnassignedGuestOrders($this->customerSession->getCustomer()->getEmail() ?: '');
    }

    /**
     * @return string
     */
    public function getAssignLink(): string
    {
        if ($this->config->isAllowCustomerToAssignItsGuestOrders() === 2) {
            $url = $this->getUrl('mfguesttocustomer/account/assignGuestOrdersToCustomerAccount');
        } else {
            $url = $this->getUrl('mfguesttocustomer/account/sendAssignLink');
        }
        return $url;
    }

    /**
     * @return string
     */
    public function toHtml(): string
    {
        return ($this->config->isEnabled() && $this->config->isAllowCustomerToAssignItsGuestOrders()) ? parent::toHtml() : '';
    }
}
