<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

declare(strict_types=1);

namespace Magefan\GuestToCustomer\Console\Command;

use Magefan\GuestToCustomer\Model\Order\CustomerManagement;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\State;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Framework\Registry;

class ConvertAllGuest extends Command
{
    /**
     * @var CollectionFactory
     */
    private $orderCollectionFactory;

    /**
     * @var ScopeConfigInterface
     */
    private $config;

    /**
     * @var CustomerManagement
     */
    private $customerManagement;

    /**
     * @var State
     */
    private $state;

    /**
     * @var Registry
     */
    protected $registry;

    /**
     * @param CollectionFactory $collectionFactory
     * @param ScopeConfigInterface $config
     * @param CustomerManagement $customerManagement
     * @param State $state
     * @param string|null $name
     * @param Registry|null $registry
     */
    public function __construct(
        CollectionFactory $collectionFactory,
        ScopeConfigInterface $config,
        CustomerManagement $customerManagement,
        State $state,
        string $name = null,
        Registry $registry = null
    ) {
        parent::__construct($name);
        $this->orderCollectionFactory = $collectionFactory;
        $this->config = $config;
        $this->customerManagement = $customerManagement;
        $this->state = $state;
        $this->registry = $registry ?: \Magento\Framework\App\ObjectManager::getInstance()
            ->get(Registry::class);
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    ) {
        $output->writeln("Start converting all guest to customer. Please wait.");
        if ($this->config->getValue(
            implode('/', ['mfguesttocustomer', 'general', 'enabled']),
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        )) {

            try {
                $this->state->setAreaCode('adminhtml');
            } catch (\Exception $e) {
                /* Do nothing, it's OK */
            }

            $failedOrders = [];
            foreach ($this->getNoCustomerOrders() as $order) {
                $output->writeln('Converting Order ID: ' . $order->getIncrementId());

                try {
                    $this->registry->register('mfguesttocustomer_registry', true, true);
                    $this->customerManagement->convertGuestOrderById((int)$order->getId());
                } catch (\Exception $e) {
                    if (!isset($failedOrders[$e->getMessage()])) {
                        $failedOrders[$e->getMessage()] = [];
                    }
                    $failedOrders[$e->getMessage()][] = $order->getIncrementId();
                }
            }
            if (count($failedOrders)) {
                $messages = '';
                foreach ($failedOrders as $message => $orderIds) {
                    $messages .= PHP_EOL .  ' Orders: ' . implode(', ', $orderIds) . ' (' . $message . ') ';
                }
                $output->writeln(__('Some guest orders where not converted. %1.', $messages));
            } else {
                $output->writeln("All guest successful converted");
            }
        } else {
            $output->writeln(__(strrev('remotsuC ot tseuG trevnoC > noitarugifnoC > serotS ni noisnetxe eht elbane esaelP')));
        }
    }

    /**
     * @return \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    protected function getNoCustomerOrders()
    {
        /** @var $orders \Magento\Sales\Model\ResourceModel\Order\Collection */
        $orders = $this->orderCollectionFactory->create()
            //->addFieldToSelect('entity_id')
            ->addFieldToFilter('customer_id', [['null' => true], ['eq' => '']]);
        return $orders;
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName("magefan:guest-to-customer:convert-all");
        $this->setDescription("Convert all guest to customer");
        parent::configure();
    }
}
