<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */
declare(strict_types=1);

namespace Magefan\GuestToCustomer\Controller\Account;

use Magento\Customer\Model\Session;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\View\Result\PageFactory;
use Magento\Framework\App\RequestInterface;
use Magefan\GuestToCustomer\Model\Order\GuestOrderManagement;
use Magefan\GuestToCustomer\Model\Config;

class AssignGuestOrdersToCustomerAccount extends \Magento\Framework\App\Action\Action
{

    /**
     * @var PageFactory
     */
    private $resultPageFactory;

    /**
     * @var RequestInterface
     */
    private $request;

    /**
     * @var GuestOrderManagement
     */
    private $guestOrderManagement;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var Session
     */
    private $customerSession;

    /**
     * @param Context $context
     * @param PageFactory $resultPageFactory
     * @param RequestInterface $request
     * @param GuestOrderManagement $guestOrderManagement
     * @param Config $config
     * @param Session $customerSession
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        RequestInterface $request,
        GuestOrderManagement $guestOrderManagement,
        Config $config,
        Session $customerSession
    ) {
        parent::__construct($context);
        $this->resultPageFactory = $resultPageFactory;
        $this->request = $request;
        $this->guestOrderManagement = $guestOrderManagement;
        $this->config = $config;
        $this->customerSession = $customerSession;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Redirect|ResultInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function execute()
    {
        if (!$this->config->isEnabled() || !$this->config->isAllowCustomerToAssignItsGuestOrders()) {
            $this->messageManager->addErrorMessage(__('This action is not available at that moment!'));
            return $this->resultRedirectFactory->create()->setPath('sales/order/history');
        }

        $customerId = (int)$this->request->getParam('id');
        if ($customerId) {
            $token = (string)$this->request->getParam('token');
            $allowEmptyToken = false;
        } else {
            $customerId =  (int) $this->customerSession->getCustomerId();
            $token = $this->guestOrderManagement->getToken($customerId) ?: '';
            $allowEmptyToken = ($this->config->isAllowCustomerToAssignItsGuestOrders() === 2);
        }

        if ($customerId && ($token || $allowEmptyToken)) {
            if ($this->guestOrderManagement->assignGuestOrdersToCustomerAccount($customerId, $token)) {
                $this->messageManager->addSuccessMessage(__('Guest order(s) successfully assigned to your account!'));
            } else {
                $this->messageManager->addErrorMessage(__('Some of your parameters are invalid, please try again later!'));
            }
        } else {
            $this->messageManager->addErrorMessage(__('Cannot assign orders. Some of the parameters are missed.'));
        }

        return $this->resultRedirectFactory->create()->setPath('sales/order/history');
    }
}
