<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */
declare(strict_types=1);

namespace Magefan\GuestToCustomer\Controller\Account;

use Magento\Framework\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;
use Magefan\GuestToCustomer\Model\Order\GuestOrderManagement;
use Magento\Customer\Model\Session;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\App\Action\Action;
use Magefan\GuestToCustomer\Model\Config;

class SendAssignLink extends Action
{
    /**
     * @var PageFactory
     */
    private $resultPageFactory;

    /**
     * @var GuestOrderManagement
     */
    private $guestOrderManagement;

    /**
     * @var Session
     */
    private $customerSession;

    /**
     * @var Config
     */
    private $config;

    /**
     * @param PageFactory $resultPageFactory
     * @param GuestOrderManagement $guestOrderManagement
     * @param Session $customerSession
     * @param Context $context
     */
    public function __construct(
        PageFactory $resultPageFactory,
        GuestOrderManagement $guestOrderManagement,
        Session $customerSession,
        Context $context,
        Config $config
    ) {
        parent::__construct($context);
        $this->resultPageFactory = $resultPageFactory;
        $this->guestOrderManagement = $guestOrderManagement;
        $this->customerSession = $customerSession;
        $this->config = $config;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        if (!$this->config->isEnabled() || !$this->config->isAllowCustomerToAssignItsGuestOrders()) {
            $this->messageManager->addErrorMessage(__('This action is not available at that moment!'));
            return $this->resultRedirectFactory->create()->setPath('sales/order/history');
        }

        if (!$this->customerSession->isLoggedIn()) {
            $this->messageManager->addErrorMessage(__('You must be logged in to execute this action!'));
            return $this->resultRedirectFactory->create()->setPath('customer/account/login');
        }

        $this->guestOrderManagement->sendAssignLink((int) $this->customerSession->getCustomerId());
        $this->messageManager->addSuccessMessage(__('A confirmation email has been sent. Please check your inbox and confirm assign action.'));

        return $this->resultFactory->create(ResultFactory::TYPE_REDIRECT)->setUrl($this->_redirect->getRefererUrl());
    }
}
