<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

namespace Magefan\GuestToCustomer\Controller\Adminhtml\All;

use Magento\Backend\App\Action\Context;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magefan\GuestToCustomer\Model\Order\CustomerManagement;
use Magento\Framework\Registry;

/**
 * Class Convert. Convert guest to customer.
 */
class Convert extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Magefan_GuestToCustomer::config_section';

    /**
     * @var CollectionFactory
     */
    protected $orderCollectionFactory;

    /**
     * @var ScopeConfigInterface
     */
    protected $config;

    /**
     * @var CustomerManagement
     */
    protected $customerManagement;

    /**
     * @var Registry
     */
    protected $registry;

    /**
     * Convert constructor.
     * @param Context $context
     * @param CollectionFactory $collectionFactory
     * @param ScopeConfigInterface $config
     * @param CustomerManagement $customerManagement
     * @param Registry|null $registry
     */
    public function __construct(
        Context $context,
        CollectionFactory $collectionFactory,
        ScopeConfigInterface $config,
        CustomerManagement $customerManagement,
        Registry $registry
    ) {
        parent::__construct($context);
        $this->orderCollectionFactory = $collectionFactory;
        $this->config = $config;
        $this->customerManagement = $customerManagement;
        $this->registry = $registry ?: \Magento\Framework\App\ObjectManager::getInstance()
            ->get(Registry::class);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface|void
     */
    public function execute()
    {
        if ($this->config->getValue(
            implode('/', ['mfguesttocustomer', 'general', 'enabled']),
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        )) {

            $failedOrders = [];
            foreach ($this->getNoCustomerOrders() as $order) {
                try {
                    $this->registry->register('mfguesttocustomer_registry', true, true);
                    $this->customerManagement->convertGuestOrderById((int)$order->getId());
                } catch (\Exception $e) {
                    if (!isset($failedOrders[$e->getMessage()])) {
                        $failedOrders[$e->getMessage()] = [];
                    }
                    $failedOrders[$e->getMessage()][] = $order->getIncrementId();
                }
            }

            if (count($failedOrders)) {
                $messages = '';
                foreach ($failedOrders as $message => $orderIds) {
                    $messages .= PHP_EOL .  ' Orders: ' . implode(', ', $orderIds) . ' (' . $message . ') ';
                }
                $this->messageManager->addExceptionMessage(
                    $e,
                    __(
                        'Some guest orders where not converted. %1.',
                        $messages
                    )
                );
            } else {
                $this->messageManager->addSuccessMessage(__('Guests converted successfully.'));
            }
        } else {
            $this->messageManager->addErrorMessage(
                __(strrev('remotsuC ot tseuG trevnoC > noitarugifnoC > serotS ni noisnetxe eht elbane esaelP'))
            );
        }

        $this->_redirect($this->_redirect->getRefererUrl());
    }

    /**
     * @return \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    protected function getNoCustomerOrders()
    {
        /** @var $orders \Magento\Sales\Model\ResourceModel\Order\Collection */
        $orders = $this->orderCollectionFactory->create()
            //->addFieldToSelect('entity_id')
            ->addFieldToFilter('customer_id', [['null' => true], ['eq' => '']]);
        return $orders;
    }
}
