<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

namespace Magefan\GuestToCustomer\Controller\Adminhtml\Order;

use Magento\Backend\App\Action\Context;
use Magefan\GuestToCustomer\Model\Order\CustomerManagement;
use Magento\Framework\Registry;

/**
 * Convert guest to customer by order id
 */
class Convert extends \Magento\Backend\App\Action
{
    /**
     * @const
     */
    const ORDER_ID_PARAM = 'order_id';

    /**
     * @var CustomerManagement
     */
    protected $customerManagement;

    /**
     * @var Registry
     */
    protected $registry;

    /**
     * Login constructor.
     * @param Context $context
     * @param CustomerManagement $customerManagement
     * @param Registry|null $registry
     */
    public function __construct(
        Context $context,
        CustomerManagement $customerManagement,
        Registry $registry = null
    ) {
        $this->customerManagement = $customerManagement;
        $this->registry = $registry ?: \Magento\Framework\App\ObjectManager::getInstance()
            ->get(Registry::class);
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface|void
     */
    public function execute()
    {
        $config = $this->_objectManager->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        if ($config->getValue(
            implode('/', ['mfguesttocustomer', 'general', 'enabled']),
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        )) {
            try {
                $orderId = $this->getRequest()->getParam('order_id');
                $this->registry->register('mfguesttocustomer_registry', true, true);
                $this->customerManagement->convertGuestOrderById($orderId);
                $this->messageManager->addSuccess(__('Customer created successfully.'));
            } catch (\Exception $e) {
                $this->messageManager->addException(
                    $e,
                    __(
                        'Something went wrong. %1.',
                        $e->getMessage()
                    )
                );
            }
        } else {
            $this->messageManager->addError(
                __(strrev('>b/<remotsuC ot tseuG trevnoC > noitarugifnoC > serotS>b< ni noisnetxe eht elbane esaelP'))
            );
        }

        $this->_redirect($this->_redirect->getRefererUrl());
    }

    /**
     * Check is allowed access
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Magefan_GuestToCustomer::config_section');
    }
}
