<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

namespace Magefan\GuestToCustomer\Model;

use Magento\Store\Model\ScopeInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Config
{
    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_ENABLED = 'mfguesttocustomer/general/enabled';

    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_AUTOCONVERT_ENABLED = 'mfguesttocustomer/general/autoconvert_order';

    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_AUTOCONVERT_ON_ORDER_STATUS = 'mfguesttocustomer/general/autoconvert_order_on_status';

    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_GROUP_ID = 'mfguesttocustomer/general/customer_group';

    /**
     * @var string
     */
    const XML_PATH_GUEST_CHANGE_ORDER_TO_CUSTOMER_GROUP_ID = 'mfguesttocustomer/general/change_order_customer_group';

    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_ASSIGN_ADDRESS = 'mfguesttocustomer/general/assign_address';

    /**
     * @var string
     */
    const XML_PATH_NOTIFICATION_MESSAGE_ENABLED = 'mfguesttocustomer/general/send_email';

    /**
     * @var string
     */
    const XML_PATH_NEWSLETTER_SUBSCRIPTION_ENABLED = 'mfguesttocustomer/general/newsletter_subscription';

    /**
     * @var string
     */
    const XML_PATH_INVALID_ADDRESS_ACTION = 'mfguesttocustomer/general/invalid_order_address_action';

    /**
     * @var string
     */
    const XML_PATH_ALLOW_CUSTOMER_TO_ASSIGN_ITS_GUEST_ORDERS = 'mfguesttocustomer/general/allow_customer_to_assign_its_guest_orders';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * Config constructor.
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param null $storeId
     * @return mixed
     */
    public function isEnabled($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_ENABLED,
            $storeId
        );
    }

    /**
     * @param null $storeId
     * @return bool
     */
    public function isAutoconvertEnabled($storeId = null): bool
    {
        return (bool)$this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_AUTOCONVERT_ENABLED,
            $storeId
        );
    }

    /**
     * @param $path
     * @param $storeId
     * @return mixed
     */
    public function getConfig($path, $storeId)
    {
        return $this->scopeConfig->getValue(
            $path,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @param null $storeId
     * @return mixed
     */
    public function getCustomerGroupId($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_GROUP_ID,
            $storeId
        );
    }

    /**
     * @param null $storedId
     * @return mixed
     */
    public function isAssignOrderAddressToCustomer($storedId = null)
    {
        return $this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_ASSIGN_ADDRESS,
            $storedId
        );
    }

    /**
     * @param null $storeId
     * @return mixed
     */
    public function isNotificationMessagesEnabled($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_NOTIFICATION_MESSAGE_ENABLED,
            $storeId
        );
    }

    /**
     * @param null $storeId
     * @return mixed
     */
    public function isNewsletterSubscriptionEnabled($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_NEWSLETTER_SUBSCRIPTION_ENABLED,
            $storeId
        );
    }

    /**
     * @param null $storeId
     * @return mixed
     */
    public function invalidOrderAddressAction($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_INVALID_ADDRESS_ACTION,
            $storeId
        );
    }

    /**
     * @param null $storedId
     * @return int
     */
    public function isAllowCustomerToAssignItsGuestOrders($storedId = null): int
    {
        return (int)$this->getConfig(
            self::XML_PATH_ALLOW_CUSTOMER_TO_ASSIGN_ITS_GUEST_ORDERS,
            $storedId
        );
    }

    /**
     * @param null $storedId
     * @return mixed
     */
    public function ifChangeOrderCustomerGroup($storedId = null)
    {
        return (bool)$this->getConfig(
            self::XML_PATH_GUEST_CHANGE_ORDER_TO_CUSTOMER_GROUP_ID,
            $storedId
        );
    }

    /**
     * @param null $storedId
     * @return array
     */
    public function getActomaticallyConvertGuestOnOrderStatus($storedId = null):array
    {
        return explode(',', $this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_AUTOCONVERT_ON_ORDER_STATUS,
            $storedId
        ));
    }
}
