<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

declare(strict_types=1);

namespace Magefan\GuestToCustomer\Model\Config\Source;

use Magento\Framework\Data\OptionSourceInterface;
use Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory as OrderStatusCollectionFactory;

/**
 * Options provider for order status list
 */
class OrderStatus implements OptionSourceInterface
{
    /**
     * Statuses
     *
     * @var OrderStatusCollectionFactory
     */
    private $orderStatusCollectionFactory;

    /**
     * Options array
     *
     * @var array
     */
    private $options;

    /**
     * @param OrderStatusCollectionFactory $orderStatusCollectionFactory
     */
    public function __construct(
        OrderStatusCollectionFactory $orderStatusCollectionFactory
    ) {
        $this->orderStatusCollectionFactory = $orderStatusCollectionFactory;
    }

    /**
     * Return options array
     *
     * @return array
     */
    public function toOptionArray():array
    {
        if (!$this->options) {
            $orderStatusCollection = $this->orderStatusCollectionFactory->create();
            $this->options = $orderStatusCollection->loadData()->toOptionArray();
            array_unshift($this->options, ['value' => 0, 'label' => __('Any Status')]);
        }
        return $this->options;
    }
}
