<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */
namespace Magefan\GuestToCustomer\Plugin\Magento\Customer\Model;

use Magento\Customer\Api\Data\CustomerInterface;
use Magefan\GuestToCustomer\Model\Config;
use Magento\Customer\Model\EmailNotification;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Registry;

class EmailNotificationPlugin
{
    /**
     * @var Config
     */
    protected $config;

    /**
     * @var Registry
     */
    protected $registry;

    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @param RequestInterface $request
     * @param Config $config
     * @param Registry|null $registry
     */
    public function __construct(
        RequestInterface $request,
        Config $config,
        Registry $registry = null
    ) {
        $this->request = $request;
        $this->config = $config;
        $this->registry = $registry ?: \Magento\Framework\App\ObjectManager::getInstance()
            ->get(Registry::class);
    }

    /**
     * @param EmailNotification $subject
     * @param callable $proceed
     * @param CustomerInterface $customer
     * @param string $type
     * @param string $backUrl
     * @param int $storeId
     * @param null $sendemailStoreId
     * @return mixed
     */
    public function aroundNewAccount(
        EmailNotification $subject,
        callable $proceed,
        CustomerInterface $customer,
        $type = 'registered',
        $backUrl = '',
        $storeId = 0,
        $sendemailStoreId = null
    ) {
        if (!$this->config->isNotificationMessagesEnabled() && $this->registry->registry('mfguesttocustomer_registry')) {
            return;
        }

        return $proceed(
            $customer,
            $type,
            $backUrl,
            $storeId,
            $sendemailStoreId
        );
    }
}
