<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

namespace Magefan\GuestToCustomer\Ui\Component\Listing\Column;

use Magento\Ui\Component\Listing\Columns\Column;
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Framework\UrlInterface;
use Magento\Framework\App\Request\Http;
use Magento\Backend\Model\Session;

/**
 * Class Assign Customer To Order Link
 */
class Assign extends Column
{
    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var Http
     */
    protected $request;

    /**
     * @var Session
     */
    protected $session;

    /**
     * @param ContextInterface $context
     * @param UiComponentFactory $uiComponentFactory
     * @param UrlInterface $urlBuilder
     * @param array $components
     * @param array $data
     */
    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        UrlInterface $urlBuilder,
        Http $request,
        Session $session,
        array $components = [],
        array $data = []
    ) {
        $this->urlBuilder = $urlBuilder;
        $this->request = $request;
        $this->session = $session;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }

    /**
     * Prepare Data Source
     *
     * @param array $dataSource
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        $orderId = (int)$this->session->getData('value')['order_id'];

        if (isset($dataSource['data']['items'])) {
            $storeId = (int)$this->context->getFilterParam('store_id');

            foreach ($dataSource['data']['items'] as &$item) {

                $href = $this->urlBuilder->getUrl(
                    'mfguesttocustomer/order/assigncustomer',
                    [
                        'order_id'    => $orderId,
                        'customer_id' => $item['entity_id'],
                        'store_id'    => $storeId,
                    ]
                );
                $item[$this->getData('name')]['edit'] = [
                    'href'   => $href,
                    'label'  => __('Assign to Order'),
                    'hidden' => false,
                ];
            }
        }

        return $dataSource;
    }
}
