<?php
/**
 *
 * BSS Commerce Co.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://bsscommerce.com/Bss-Commerce-License.txt
 *
 * @category  BSS
 * @package   Bss_ProductLabelGraphQl
 * @author    Extension Team
 * @copyright Copyright (c) 2020 BSS Commerce Co. ( http://bsscommerce.com )
 * @license   http://bsscommerce.com/Bss-Commerce-License.txt
 */
namespace Bss\ProductLabelGraphQl\Helper;

use Bss\ProductLabel\Model\LabelFactory;
use Bss\ProductLabel\Model\ResourceModel\Label;
use Exception;
use Magento\Catalog\Model\Product;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Data to get data
 */
class Data extends AbstractHelper
{
    const IS_ENABLE_MODULE = "productlabel/general/enable";
    const IS_DISPLAY_MULTIPLE_LABEL = "productlabel/general/display_multiple_label";
    const IS_ENABLE_ONLY_OUT_OF_STOCK_LABEL = "productlabel/general/display_only_out_of_stock_label";
    const IS_NOT_DISPLAY_ON = "productlabel/general/not_display_label_on";
    const SELECT_PRODUCT_LIST = "productlabel/display/product_list";
    const SELECTOR_PRODUCT_PAGE = "productlabel/display/product_page";
    /**
     * @var StoreManagerInterface
     */
    protected $storeManagerInterface;
    /**
     * @var LabelFactory
     */
    protected $labelFatory;
    /**
     * @var Json
     */
    protected $json;
    /**
     * @var Label
     */
    protected $labelResource;

    /**
     * Data constructor.
     *
     * @param StoreManagerInterface $storeManagerInterface
     * @param Context $context
     * @param Json $json
     * @param LabelFactory $labelFactory
     * @param Label $labelResource
     */
    public function __construct(
        StoreManagerInterface $storeManagerInterface,
        Context $context,
        Json $json,
        LabelFactory $labelFactory,
        Label $labelResource
    ) {
        $this->storeManagerInterface = $storeManagerInterface;
        $this->labelFatory = $labelFactory;
        $this->json = $json;
        $this->labelResource = $labelResource;
        parent::__construct($context);
    }

    /**
     * Get config
     *
     * @param string $path
     * @param string $storeView
     * @return mixed
     */
    public function getConfig($path, $storeView)
    {
        return $this->scopeConfig->getValue($path, ScopeInterface::SCOPE_STORE, $storeView);
    }

    /**
     * Get url media
     *
     * @return string
     * @throws NoSuchEntityException
     */
    public function getMediaUrl()
    {
        return $this->storeManagerInterface->getStore()->getBaseUrl(
            UrlInterface::URL_TYPE_MEDIA
        );
    }

    /**
     * Get product label by id
     *
     * @param int $id
     * @return mixed|null
     * @throws NoSuchEntityException
     */
    public function getProductLabelById($id)
    {
        $data=[];
        try {
            $label = $this->labelFatory->create()->load($id);
            $data = $label->getData();
        } catch (Exception $exception) {
            throw (__($exception->getMessage()));
        }
        if (isset($data["conditions_serialized"]) && !empty($data["conditions_serialized"])) {
            $data["file"] = $this->getMediaUrl() . $data['image'];
            $data["image_data"] = $this->json->unserialize($data["image_data"]);
            $data["conditions_serialized"] = $this->json->unserialize($data["conditions_serialized"]);
            if (isset($data["conditions_serialized"]["conditions"])) {
                $data["conditions_serialized"]["conditions"] =
                    $this->json->serialize($data["conditions_serialized"]["conditions"]);
            }
        }
        return $data;
    }

    /**
     * Get product label by product label
     *
     * @param Product $product
     * @return array
     * @throws \Zend_Db_Statement_Exception
     */
    public function getLabelByProductId($product)
    {
        $labelDatas = $this->labelResource->getIndexedProduct($product->getId());
        $data = [];
        if (!empty($labelDatas)) {
            try {
                foreach ($labelDatas as $labelData) {
                    if (empty($labelData)) {
                        continue;
                    }
                    $labelData["id"] = $labelData["label_id"];
                    $label = $this->labelFatory->create()->load($labelData["id"]);
                    $labelData["name"] = $label->getName();
                    $labelData["file"] = $this->getMediaUrl() . $labelData['image'];
                    $labelData["image_data"] = $this->json->unserialize($labelData['image_data']);
                    $labelData["conditions_serialized"] = $this->json->unserialize($label->getConditionsSerialized());
                    if (isset($labelData["conditions_serialized"]["conditions"])) {
                        $labelData["conditions_serialized"]["conditions"] =
                            $this->json->serialize($labelData["conditions_serialized"]["conditions"]);
                    }
                    $labelData["apply_outofstock_product"] = $label->getApplyOutofstockProduct();
                    $labelData["active"] = $label->getActive();
                    $labelData["created_at"] = $label->getCreatedAt();
                    $labelData["updated_at"] = $label->getUpdatedAt();
                    $data[] = $labelData;
                }
            } catch (Exception $exception) {
                $this->_logger->critical($exception);
            }
            usort($data, [$this, 'customUsort']);
        }

        return $data;
    }

    /**
     * Sort product label by priority
     *
     * @param array $a
     * @param array $b
     * @return int
     */
    protected function customUsort($a, $b)
    {
        if ($a['priority'] == $b['priority']) {
            return 0;
        }
        return ($a['priority'] < $b['priority']) ? 1 : -1;
    }
}
