<?php

namespace Cisin\Salesordergrid\Ui\Component\Listing\Column;

use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Store\Model\StoreManagerInterface;  // Correctly import StoreManager
use Magento\Sales\Model\OrderFactory;
use Magento\Framework\Pricing\Helper\Data as PriceHelper;

/**
 * @api
 * @since 100.0.2
 */
class Brand extends \Magento\Ui\Component\Listing\Columns\Column
{
    protected $orderFactory;
    protected $priceHelper;
    protected $storeManager;

    /**
     * @param ContextInterface $context
     * @param UiComponentFactory $uiComponentFactory
     * @param OrderFactory $orderFactory
     * @param PriceHelper $priceHelper
     * @param StoreManagerInterface $storeManager
     * @param array $components
     * @param array $data
     */
    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        OrderFactory $orderFactory, // Correct the order here
        PriceHelper $priceHelper,
        StoreManagerInterface $storeManager, // StoreManager should be after PriceHelper
        array $components = [],
        array $data = []
    ) {
        parent::__construct($context, $uiComponentFactory, $components, $data);
        $this->orderFactory = $orderFactory;
        $this->priceHelper = $priceHelper;
        $this->storeManager = $storeManager;
    }

    /**
     * Prepare Data Source
     *
     * @param array $dataSource
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        if (isset($dataSource['data']['items'])) {
            foreach ($dataSource['data']['items'] as &$item) {
                $brand_name = '';
                $count = 1;

                $order = $this->orderFactory->create()->load($item['entity_id']);
                $orderItems = $order->getAllItems();

                foreach ($orderItems as $data) {
                    $order_item = $data->getData();
                    $product = $this->loadProduct($data->getProductId());

                    if ($product) {
                        $brand_name .= $count . ". " . $this->getBrandFromProduct($product) . "<br/>";
                    } else {
                        $brand_name .= $count . ". " . "Product not found" . "<br/>";
                    }
                    $count++;
                }

                $item['brand'] = $brand_name;
            }
        }

        return $dataSource;
    }

    /**
     * Load Product by ID
     *
     * @param int $productId
     * @return \Magento\Catalog\Model\Product|null
     */
    protected function loadProduct($productId)
    {
        $product = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Product')->load($productId);
        return $product && $product->getId() ? $product : null;
    }

    /**
     * Get Brand from Product
     *
     * @param \Magento\Catalog\Model\Product $product
     * @return string
     */
    protected function getBrandFromProduct($product)
    {
        if ($product->getId() && $product->getData('brand')) {
            return $product->getAttributeText('brand') ?: 'No brand available';
        }

        return 'Brand not available';
    }
}
