<?php

namespace MagePsycho\MassImporterPro\Block\Adminhtml;

use Magento\Backend\Block\Widget\Button\SplitButton;
use Magento\Backend\Block\Widget\Container;
use Magento\Backend\Block\Widget\Context;
use MagePsycho\MassImporterPro\Model\System\Config\Source\EntityType;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Import extends Container
{
    /**
     * @var EntityType
     */
    private $entityType;

    public function __construct(
        Context $context,
        EntityType $entityType,
        array $data = []
    ) {
        $this->entityType = $entityType;
        parent::__construct($context, $data);
    }

    protected function _prepareLayout()
    {
        $exportButtonProps = [
            'id'            => 'add_new_export',
            'label'         => __('New Export'),
            'class'         => 'secondary',
            'button_class'  => '',
            'class_name'    => SplitButton::class,
            'options'       => $this->_getAddExportButtonOptions(),
        ];
        $this->buttonList->add('add_new_export', $exportButtonProps);

        $importButtonProps = [
            'id'            => 'add_new_import',
            'label'         => __('New Import'),
            'class'         => 'add',
            'button_class'  => '',
            'class_name'    => SplitButton::class,
            'options'       => $this->getAddImportButtonOptions(),
        ];
        $this->buttonList->add('add_new_import', $importButtonProps);

        return parent::_prepareLayout();
    }

    /**
     * Retrieve options for 'New Import' split button
     *
     * @return array
     */
    protected function getAddImportButtonOptions()
    {
        $splitButtonOptions = [];
        $types = $this->entityType->toOptionHash(false);

        foreach ($types as $typeId => $label) {
            $splitButtonOptions[$typeId] = [
                'label' => __($label),
                'onclick' => "setLocation('" . $this->_getImportUrl($typeId) . "')",
                'default' => 'pricing' == $typeId,
            ];
        }

        return $splitButtonOptions;
    }

    /**
     * Retrieve options for 'New Export' split button
     *
     * @return array
     */
    protected function _getAddExportButtonOptions()
    {
        $splitButtonOptions = [];
        $types = $this->entityType->toOptionHash(false);

        foreach ($types as $typeId => $label) {
            $splitButtonOptions[$typeId] = [
                'label' => __($label),
                'onclick' => "setLocation('" . $this->_getExportUrl($typeId) . "')",
                'default' => 'pricing' == $typeId,
            ];
        }

        return $splitButtonOptions;
    }

    /**
     * Retrieve import url by specified entity type
     *
     * @param string $entityType
     * @return string
     */
    protected function _getImportUrl($entityType)
    {
        return $this->getUrl(
            'magepsycho_massimporterpro/import/new',
            ['file_type' => 'csv', 'entity' => $entityType]
        );
    }

    /**
     * Retrieve export url by specified entity type
     *
     * @param string $entityType
     * @return string
     */
    protected function _getExportUrl($entityType)
    {
        return $this->getUrl(
            'adminhtml/export/index',
            ['entity' => $entityType]
        );
    }

    /**
     * Check whether it is single store mode
     *
     * @return bool
     */
    public function isSingleStoreMode()
    {
        return $this->_storeManager->isSingleStoreMode();
    }
}
