<?php

namespace MagePsycho\MassImporterPro\Controller\Adminhtml\Import;

use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use MagePsycho\MassImporterPro\Helper\Data as MipHelper;
use MagePsycho\MassImporterPro\Model\System\Config\Source\EntityType as MipEntityType;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Edit extends \MagePsycho\MassImporterPro\Controller\Adminhtml\Import
{
    /**
     * @var MipHelper
     */
    private $mipHelper;

    public function __construct(
        Context $context,
        MipHelper $mipHelper
    ) {
        parent::__construct($context);
        $this->mipHelper = $mipHelper;
    }

    /**
     * Index action
     *
     * @return \Magento\Backend\Model\View\Result\Page
     */
    public function execute()
    {
        if (!$this->mipHelper->isActive() || !$this->mipHelper->isValid()) {
            $this->messageManager->addComplexWarningMessage(
                'mpMassImporterProComplexMessage',
                [
                    'message' => (string) __(
                        'Please check if the extension is active. You can activate it from <a href="%1">%2</a>.',
                        $this->getUrl('adminhtml/system_config/edit/section/magepsycho_massimporterpro'),
                        'STORES » Configuration » MAGEPSYCHO » Mass Importer Pro'
                    )
                ]
            );
        }

        $entityTypeConfig = $this->_objectManager->get(MipEntityType::class);
        $entity = $entityTypeConfig->getOptionText($this->_request->getParam('entity'));

        /** @var \Magento\Backend\Model\View\Result\Page $resultPage */
        $resultPage = $this->resultFactory->create(ResultFactory::TYPE_PAGE);
        $resultPage->setActiveMenu('MagePsycho_MassImporterPro::import');
        $resultPage->getConfig()->getTitle()->prepend(__('Import/Export'));
        $resultPage->getConfig()->getTitle()->prepend(__('Import %1', $entity));
        $resultPage->addBreadcrumb(__('Import'), __('Import'));
        return $resultPage;
    }
}
