<?php

namespace MagePsycho\MassImporterPro\Controller\Adminhtml\Import;

use MagePsycho\MassImporterPro\Controller\Adminhtml\ImportResult as ImportResultController;
use Magento\Framework\Controller\ResultFactory;
use MagePsycho\MassImporterPro\Model\Import;
use Magento\Framework\Filesystem\Io\File;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Start extends ImportResultController
{
    /**
     * @var \MagePsycho\MassImporterPro\Model\Import
     */
    protected $importModel;

    /**
     * @var \Magento\Framework\Message\ExceptionMessageFactoryInterface
     */
    private $exceptionMessageFactory;

    /**
     * @var File
     */
    private $fileIo;

    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \MagePsycho\MassImporterPro\Model\Report\ReportProcessorInterface $reportProcessor,
        \MagePsycho\MassImporterPro\Model\History $historyModel,
        \MagePsycho\MassImporterPro\Helper\Report $reportHelper,
        \Magento\Framework\HTTP\PhpEnvironment\Request $request,
        \MagePsycho\MassImporterPro\Model\Import $importModel,
        \Magento\Framework\Message\ExceptionMessageFactoryInterface $exceptionMessageFactory,
        File $fileIo
    ) {
        parent::__construct($context, $reportProcessor, $historyModel, $reportHelper, $request);
        $this->importModel = $importModel;
        $this->exceptionMessageFactory = $exceptionMessageFactory;
        $this->fileIo = $fileIo;
    }

    /**
     * Start import process action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        if ($data) {
            /** @var \Magento\Framework\View\Result\Layout $resultLayout */
            $resultLayout = $this->resultFactory->create(ResultFactory::TYPE_LAYOUT);

            /** @var $resultBlock \MagePsycho\MassImporterPro\Block\Adminhtml\Import\Frame\Result */
            $resultBlock = $resultLayout->getLayout()->getBlock('import.frame.result');
            $resultBlock
                ->addAction('show', 'import_validation_container')
                ->addAction('innerHTML', 'import_validation_container_header', __('Status'))
                ->addAction('hide', ['edit_form', 'upload_button', 'messages']);

            $this->importModel->setData($data);
            $errorAggregator = $this->importModel->getErrorAggregator();
            $errorAggregator->initValidationStrategy(
                $this->importModel->getData(Import::FIELD_NAME_VALIDATION_STRATEGY),
                $this->importModel->getData(Import::FIELD_NAME_ALLOWED_ERROR_COUNT)
            );

            try {
                $this->importModel->importSource();
            } catch (\Exception $e) {
                $resultMessageBlock = $resultLayout->getLayout()->getBlock('messages');
                $message = $this->exceptionMessageFactory->createMessage($e);
                $html = $resultMessageBlock->addMessage($message)->toHtml();
                $errorAggregator->addError(
                    \MagePsycho\MassImporterPro\Model\Import\Entity\AbstractEntity::ERROR_CODE_SYSTEM_EXCEPTION,
                    \MagePsycho\MassImporterPro\Model\Import\ErrorProcessing\ProcessingError::ERROR_LEVEL_CRITICAL,
                    null,
                    null,
                    null,
                    $html
                );
            }

            if ($this->importModel->getErrorAggregator()->hasToBeTerminated()) {
                $resultBlock->addError(__('Maximum error count has been reached or system error is occurred!'));
                $this->addErrorMessages($resultBlock, $errorAggregator);
            } else {
                // Silently delete the uploaded file via ajax
                $result = $data[\MagePsycho\MassImporterPro\Model\Import::FIELD_NAME_SOURCE_FILE][0];
                $uploadedFile = $result['path'] . '/' . $result['file'];
                if ($this->fileIo->fileExists($uploadedFile)) {
                    $this->fileIo->rm($uploadedFile);
                }

                $this->importModel->invalidateIndex();

                $noticeHtml = $this->historyModel->getSummary();

                if ($this->historyModel->getErrorFile()) {
                    $noticeHtml .=  '<div class="import-error-wrapper">' . __('Only the first 100 errors are shown. ')
                                    . '<a href="'
                                    . $this->createDownloadUrlImportHistoryFile($this->historyModel->getErrorFile())
                                    . '">' . __('Download full report') . '</a></div>';
                }

                $resultBlock->addNotice(
                    $noticeHtml
                );

                $this->addErrorMessages($resultBlock, $errorAggregator);
                $resultBlock->addSuccess(__('Import successfully done'));
            }

            return $resultLayout;
        }

        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $resultRedirect->setPath('magepsycho_massimporterpro/*/index');
        return $resultRedirect;
    }
}
