<?php

namespace MagePsycho\MassImporterPro\Controller\Adminhtml\Import;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use MagePsycho\MassImporterPro\Model\Uploader;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Upload extends Action
{
    /**
     * @var string
     */
    const ACTION_RESOURCE = 'MagePsycho_MassImporterPro::import';

    /**
     * uploader
     *
     * @var Uploader
     */
    protected $uploader;

    /**
     * @param Context $context
     * @param Uploader $uploader
     */
    public function __construct(
        Context $context,
        Uploader $uploader
    ) {
        parent::__construct($context);
        $this->uploader = $uploader;
    }

    /**
     * Upload file controller action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        try {
            $this->uploader->initBasePaths($this->_request->getParam('entity'));
            $result = $this->uploader->saveFileToTmpDir('import_file');

            $result['cookie'] = [
                'name'      => $this->_getSession()->getName(),
                'value'     => $this->_getSession()->getSessionId(),
                'lifetime'  => $this->_getSession()->getCookieLifetime(),
                'path'      => $this->_getSession()->getCookiePath(),
                'domain'    => $this->_getSession()->getCookieDomain(),
            ];
        } catch (\Exception $e) {
            $result = [
                'error'     => $e->getMessage(),
                'errorcode' => $e->getCode()
            ];
        }
        return $this->resultFactory->create(ResultFactory::TYPE_JSON)->setData($result);
    }
}
