<?php

namespace MagePsycho\MassImporterPro\Model;

use Magento\Framework\ObjectManagerInterface;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class UploaderPool
{
    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;
    /**
     * @var array
     */
    protected $uploaders;

    /**
     * @param ObjectManagerInterface $objectManager
     * @param array $uploaders
     */
    public function __construct(
        ObjectManagerInterface $objectManager,
        array $uploaders = []
    ) {
        $this->objectManager = $objectManager;
        $this->uploaders     = $uploaders;
    }

    /**
     * @param $type
     * @return Uploader
     * @throws \Exception
     */
    public function getUploader($type)
    {
        if (!isset($this->uploaders[$type])) {
            throw new \Magento\Framework\Exception\LocalizedException(
                __('Uploader not found for type: %1', $type)
            );
        }
        if (!is_object($this->uploaders[$type])) {
            $this->uploaders[$type] = $this->objectManager->create($this->uploaders[$type]);
        }

        $uploader = $this->uploaders[$type];
        if (!($uploader instanceof Uploader)) {
            throw new \Magento\Framework\Exception\LocalizedException(
                __('Uploader for type %1 not instance of %2', $type, Uploader::class)
            );
        }
        return $uploader;
    }
}
