<?php

namespace MagePsycho\MassImporterPro\Observer\Adminhtml;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class ControllerActionPredispatch implements ObserverInterface
{
    /**
     * @var \MagePsycho\MassImporterPro\Helper\Data
     */
    protected $massImporterProHelper;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $messageManager;

    /**
     * ControllerActionPredispatch constructor.
     *
     * @param \MagePsycho\MassImporterPro\Helper\Data $massImporterProHelper
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     */
    public function __construct(
        \MagePsycho\MassImporterPro\Helper\Data $massImporterProHelper,
        \Magento\Framework\Message\ManagerInterface $messageManager
    ) {
        $this->massImporterProHelper   = $massImporterProHelper;
        $this->messageManager       = $messageManager;
    }

    public function execute(Observer $observer)
    {
        $isValid          = $this->massImporterProHelper->isValid();
        $isActive         = $this->massImporterProHelper->isActive();
        $request          = $observer->getRequest();
        $fullActionName   = $request->getFullActionName();
        if ($isActive
            && !$isValid
            && 'adminhtml_system_config_edit' == $fullActionName
            && 'magepsycho_massimporterpro' == $request->getParam('section')
        ) {
            $this->messageManager->addComplexErrorMessage(
                'mpMassImporterProComplexMessage',
                [
                    'message' => $this->massImporterProHelper->getMessage()
                ]
            );
        }
        return $this;
    }
}
