<?php

namespace MagePsycho\MassImporterPro\Ui\Component\Listing\Column;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class ImportAction extends \Magento\Ui\Component\Listing\Columns\Column
{
    private const URL_PATH_EDIT = 'magepsycho_massimporterpro/history/edit';
    private const URL_PATH_VIEW_LOG = 'magepsycho_massimporterpro/history/viewLog';
    private const URL_PATH_DOWNLOAD_FILE = 'magepsycho_massimporterpro/history/download';
    private const URL_PATH_DELETE = 'magepsycho_massimporterpro/history/delete';

    /**
     * URL builder
     *
     * @var \Magento\Framework\UrlInterface
     */
    public $urlBuilder;

    public function __construct(
        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Framework\View\Element\UiComponent\ContextInterface $context,
        \Magento\Framework\View\Element\UiComponentFactory $uiComponentFactory,
        array $components = [],
        array $data = []
    ) {
        $this->urlBuilder = $urlBuilder;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }

    /**
     * Prepare Data Source
     *
     * @param array $dataSource
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        if (isset($dataSource['data']['items'])) {
            foreach ($dataSource['data']['items'] as & $item) {
                if (isset($item['entity_id'])) {
                    $item[$this->getData('name')] = [
                        'delete' => [
                            'href' => $this->urlBuilder->getUrl(
                                static::URL_PATH_DELETE,
                                [
                                    'id' => $item['entity_id']
                                ]
                            ),
                            'label' => __('Delete'),
                            'confirm' => [
                                'title'   => __('Delete'),
                                'message' => __('Are you sure you wan\'t to delete the entry?')
                            ]
                        ],
                        'download_import_file' => [
                            'href' => $this->urlBuilder->getUrl(
                                static::URL_PATH_DOWNLOAD_FILE,
                                [
                                    'filename' => $item['imported_file']
                                ]
                            ),
                            'label' => __('Download (Imported File)')
                        ],
                    ];
                    if ($item['error_file'] != '') {
                        $item[$this->getData('name')] = array_merge($item[$this->getData('name')], [
                            'download_error_report' => [
                                'href' => $this->urlBuilder->getUrl(
                                    static::URL_PATH_DOWNLOAD_FILE,
                                    [
                                        'filename' => $item['error_file']
                                    ]
                                ),
                                'label' => __('Download (Error Report)')
                            ],
                        ]);
                    }
                }
            }
        }
        return $dataSource;
    }
}
