<?php

namespace MagePsycho\MipPricing\Model\Import\Pricing\Validator;

use MagePsycho\MipPricing\Model\Import\Pricing\RowValidatorInterface;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
abstract class AbstractPrice extends AbstractImportValidator implements RowValidatorInterface
{
    const INCREMENT_OPERATOR = '+';
    const DECREMENT_OPERATOR = '-';
    const PERCENTAGE_SIGN    = '%';

    /**
     * @var \Magento\Customer\Api\GroupRepositoryInterface
     */
    protected $groupRepository;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * Customer groups ID-to-name.
     *
     * @var array
     */
    protected $customerGroups = [];

    /**
     * @param \Magento\Customer\Api\GroupRepositoryInterface $groupRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        \Magento\Customer\Api\GroupRepositoryInterface $groupRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
    ) {
        $this->groupRepository = $groupRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    /**
     * {@inheritdoc}
     */
    public function init($context)
    {
        foreach ($this->groupRepository->getList($this->searchCriteriaBuilder->create())->getItems() as $group) {
            $this->customerGroups[$group->getId()] = $group->getCode();
        }
        return parent::init($context);
    }

    /**
     * Get existing customers groups
     *
     * @return array
     */
    public function getCustomerGroups()
    {
        if (!$this->customerGroups) {
            $this->init($this->context);
        }
        return $this->customerGroups;
    }

    /**
     * Supports different price formats:
     * +10%
     * -10%
     * +10
     * -10
     * 10%
     * 10
     * 10.50
     * etc.
     *
     * @param $value
     * @return false|int
     */
    public function assertPriceFormat($value)
    {
        if (strtolower($value) == self::DELETE_MARKER) {
            return true;
        }
        return preg_match('/^[+-]?(\d+\.)?(\d+)[%]?$/', $value);
    }
}
