<?php

namespace MagePsycho\MipPricing\Plugin\Controller\Adminhtml\Export;

use Magento\AdvancedPricingImportExport\Model\Export\AdvancedPricing as ExportAdvancedPricing;
use MagePsycho\MipPricing\Model\Export\Pricing as ExportMipPricing;
use Magento\Catalog\Model\Product as CatalogProduct;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class GetFilter
{
    /**
     * @var \Magento\Framework\Controller\ResultFactory
     */
    protected $resultFactory;

    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    protected $objectManager;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $messageManager;

    public function __construct(
        Context $context
    ) {
        $this->resultFactory    = $context->getResultFactory();
        $this->objectManager    = $context->getObjectManager();
        $this->messageManager   = $context->getMessageManager();
    }

    public function aroundExecute(
        \Magento\AdvancedPricingImportExport\Controller\Adminhtml\Export\GetFilter $subject,
        \Closure $proceed
    ) {
        $data = $subject->getRequest()->getParams();
        if ($subject->getRequest()->isXmlHttpRequest() && $data) {
            try {
                if (in_array(
                        $data['entity'],
                        [
                            ExportAdvancedPricing::ENTITY_ADVANCED_PRICING,
                            ExportMipPricing::ENTITY_MIP_PRICING
                        ]
                    )
                ) {
                    $data['entity'] = CatalogProduct::ENTITY;
                }
                /** @var \Magento\Framework\View\Result\Layout $resultLayout */
                $resultLayout = $this->resultFactory->create(ResultFactory::TYPE_LAYOUT);
                /** @var $attrFilterBlock \Magento\ImportExport\Block\Adminhtml\Export\Filter */
                $attrFilterBlock = $resultLayout->getLayout()->getBlock('export.filter');
                /** @var $export \Magento\ImportExport\Model\Export */
                $export = $this->objectManager->create(\Magento\ImportExport\Model\Export::class);
                $export->setData($data);
                $export->filterAttributeCollection(
                    $attrFilterBlock->prepareCollection($export->getEntityAttributeCollection())
                );
                return $resultLayout;
            } catch (\Exception $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            }
        } else {
            $this->messageManager->addErrorMessage(__('Please correct the data sent.'));
        }
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $resultRedirect->setPath('adminhtml/*/index');
        return $resultRedirect;
    }
}
