<?php

namespace UdeyTech\CategoryHelper\Setup\Patch\Data;

use Magento\Eav\Setup\EavSetupFactory;
use Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Catalog\Model\Product;

class AddFaqAttribute implements DataPatchInterface
{
    private $moduleDataSetup;
    private $eavSetupFactory;

    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        EavSetupFactory $eavSetupFactory
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->eavSetupFactory = $eavSetupFactory;
    }

    public function apply()
    {
        /** @var \Magento\Eav\Setup\EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);

        $eavSetup->addAttribute(
            Product::ENTITY,
            'product_faq',
            [
                'type' => 'text',
                'label' => 'Question And Answer',
                'input' => 'textarea',
                'backend' => '',
                'frontend' => '',
                'required' => false,
                'visible_on_front' => false,
                'user_defined' => true,
                'global' => ScopedAttributeInterface::SCOPE_STORE,
                'visible' => true,
                'group' => 'Content',
                'apply_to' => '',
                'wysiwyg_enabled' => true,
                'is_html_allowed_on_front' => true
            ]
        );

        // Add product_dimensions attribute
        $eavSetup->addAttribute(
            Product::ENTITY,
            'product_dimensions',
            [
                'type' => 'text',
                'label' => 'Product Dimensions',
                'input' => 'textarea',
                'backend' => '',
                'frontend' => '',
                'required' => false,
                'visible_on_front' => false,
                'user_defined' => true,
                'global' => ScopedAttributeInterface::SCOPE_STORE,
                'visible' => true,
                'group' => 'Content',
                'apply_to' => '',
                'wysiwyg_enabled' => true,
                'is_html_allowed_on_front' => true
            ]
        );
    }

    public static function getDependencies()
    {
        return [];
    }

    public function getAliases()
    {
        return [];
    }
}
