<?php

namespace MagePsycho\MassImporterPro\Block\Adminhtml\Import\Edit;

use MagePsycho\MassImporterPro\Model\Import;
use MagePsycho\MassImporterPro\Model\Import\ErrorProcessing\ProcessingErrorAggregatorInterface;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Form extends \Magento\Backend\Block\Widget\Form\Generic
{
    /**
     * Basic import model
     *
     * @var \MagePsycho\MassImporterPro\Model\Import
     */
    protected $_importModel;

    /**
     * @var \MagePsycho\MassImporterPro\Model\Source\Import\EntityFactory
     */
    protected $_entityFactory;

    /**
     * @var \MagePsycho\MassImporterPro\Model\Source\Import\Behavior\Factory
     */
    protected $_behaviorFactory;

    /**
     * @var \MagePsycho\MassImporterPro\Model\System\Config\Source\ImportSource
     */
    private $importSourceConfig;

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \MagePsycho\MassImporterPro\Model\Import $importModel,
        \MagePsycho\MassImporterPro\Model\Source\Import\EntityFactory $entityFactory,
        \MagePsycho\MassImporterPro\Model\Source\Import\Behavior\Factory $behaviorFactory,
        \MagePsycho\MassImporterPro\Model\System\Config\Source\ImportSource $importSourceConfig,
        array $data = []
    ) {
        $this->_entityFactory = $entityFactory;
        $this->_behaviorFactory = $behaviorFactory;
        parent::__construct($context, $registry, $formFactory, $data);
        $this->_importModel = $importModel;
        $this->importSourceConfig = $importSourceConfig;
    }

    /**
     * Add fieldsets
     *
     * @return $this
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    protected function _prepareForm()
    {
        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create(
            [
                'data' => [
                    'id' => 'edit_form',
                    'action' => $this->getUrl('magepsycho_massimporterpro/*/validate'),
                    'method' => 'post',
                    'enctype' => 'multipart/form-data',
                ],
            ]
        );

        // base fieldset
        $fieldsets['base'] = $form->addFieldset('base_fieldset', ['legend' => __('Import Settings')]);
        $entityTypeDisabled         = false;
        $entityTypeAfterElementHtml = $this->getDownloadSampleFileHtml();
        if ($entity = $this->_request->getParam('entity')) {
            $entityTypeDisabled = true;
            $entityTypeAfterElementHtml .= '<input type="hidden" name="entity" id="entity" value="' . $entity . '" />';
        }
        $fieldsets['base']->addField(
            'entity',
            'select',
            [
                'name' => 'entity',
                'title' => __('Entity Type'),
                'label' => __('Entity Type'),
                'required' => true,
                'disabled' => $entityTypeDisabled,
                'onchange' => 'mipImporter.handleEntityTypeSelector();',
                'values' => $this->_entityFactory->create()->toOptionArray(),
                'after_element_html' => $entityTypeAfterElementHtml,
            ]
        );

        // add behaviour fieldsets
        $uniqueBehaviors = $this->_importModel->getUniqueEntityBehaviors();
        foreach ($uniqueBehaviors as $behaviorCode => $behaviorClass) {
            $fieldsets[$behaviorCode] = $form->addFieldset(
                $behaviorCode . '_fieldset',
                ['legend' => __('Import Behavior'), 'class' => 'no-display']
            );
            /** @var $behaviorSource \MagePsycho\MassImporterPro\Model\Source\Import\AbstractBehavior */
            $fieldsets[$behaviorCode]->addField(
                $behaviorCode,
                'select',
                [
                    'name' => 'behavior',
                    'title' => __('Import Behavior'),
                    'label' => __('Import Behavior'),
                    'required' => true,
                    'disabled' => true,
                    'values' => $this->_behaviorFactory->create($behaviorClass)->toOptionArray(),
                    'class' => $behaviorCode,
                    'onchange' => 'mipImporter.handleImportBehaviorSelector();',
                    'note' => ' ',
                    'after_element_html' => $this->getBehaviorInfoToolTip()
                ]
            );
            $fieldsets[$behaviorCode]->addField(
                $behaviorCode . \MagePsycho\MassImporterPro\Model\Import::FIELD_NAME_VALIDATION_STRATEGY,
                'select',
                [
                    'name' => \MagePsycho\MassImporterPro\Model\Import::FIELD_NAME_VALIDATION_STRATEGY,
                    'title' => __('Validation Strategy'),
                    'label' => __('Validation Strategy'),
                    'required' => true,
                    'class' => $behaviorCode,
                    'disabled' => true,
                    'values' => [
                        ProcessingErrorAggregatorInterface::VALIDATION_STRATEGY_STOP_ON_ERROR => __('Stop on Error'),
                        ProcessingErrorAggregatorInterface::VALIDATION_STRATEGY_SKIP_ERRORS => __('Skip error entries')
                    ],
                    'after_element_html' => $this->getValidationStrategyInfoToolTip(),
                ]
            );
            $fieldsets[$behaviorCode]->addField(
                $behaviorCode . '_' . \MagePsycho\MassImporterPro\Model\Import::FIELD_NAME_ALLOWED_ERROR_COUNT,
                'text',
                [
                    'name' => \MagePsycho\MassImporterPro\Model\Import::FIELD_NAME_ALLOWED_ERROR_COUNT,
                    'label' => __('Allowed Errors Count'),
                    'title' => __('Allowed Errors Count'),
                    'required' => true,
                    'disabled' => true,
                    'value' => 10,
                    'class' => $behaviorCode . ' validate-number validate-greater-than-zero input-text',
                    'after_element_html' => $this->getAllowedErrorsCountInfoToolTip(),
                ]
            );
            $fieldsets[$behaviorCode]->addField(
                $behaviorCode . '_' . \MagePsycho\MassImporterPro\Model\Import::FIELD_FIELD_SEPARATOR,
                'hidden',
                [
                    'name' => \MagePsycho\MassImporterPro\Model\Import::FIELD_FIELD_SEPARATOR,
                    'label' => __('Field separator'),
                    'title' => __('Field separator'),
                    'required' => true,
                    'disabled' => true,
                    'class' => $behaviorCode,
                    'value' => ',',
                ]
            );
            $fieldsets[$behaviorCode]->addField(
                $behaviorCode . \MagePsycho\MassImporterPro\Model\Import::FIELD_FIELD_MULTIPLE_VALUE_SEPARATOR,
                'hidden',
                [
                    'name' => \MagePsycho\MassImporterPro\Model\Import::FIELD_FIELD_MULTIPLE_VALUE_SEPARATOR,
                    'label' => __('Multiple value separator'),
                    'title' => __('Multiple value separator'),
                    'required' => true,
                    'disabled' => true,
                    'class' => $behaviorCode,
                    'value' => Import::DEFAULT_GLOBAL_MULTI_VALUE_SEPARATOR,
                ]
            );

        }

        // fieldset for file uploading
        $fieldsets['upload'] = $form->addFieldset(
            'upload_file_fieldset',
            ['legend' => __('Import Source'), 'class' => 'no-display']
        );

        $importViaInputHtml = sprintf(
            '<input type="hidden" name="imported_via" id="imported_via" value="%s" />',
            \MagePsycho\MassImporterPro\Model\System\Config\Source\ImportMethod::IMPORT_METHOD_WEB
        );
        $fieldsets['upload']->addField(
            'import_source',
            'select',
            [
                'name' => 'import_source',
                'label' => __('Import Source'),
                'title' => __('Import Source'),
                'required' => true,
                'class' => 'input-text',
                'onchange' => 'mipImporter.handleImportSourceSelector();',
                'values' => $this->importSourceConfig->getAllOptions(),
            ]
        )->setAfterElementHtml($this->getCsvFileInfoToolTip() . $importViaInputHtml);

        $fieldsets['upload']->addField(
            'drag_n_drop_upload',
            'text',
            [
                'name'         => 'drag_n_drop_upload',
                'label'        => __("Upload via Ajax"),
            ]
        );
        $form->getElement('drag_n_drop_upload')->setRenderer(
            $this->getLayout()->createBlock(
                \MagePsycho\MassImporterPro\Block\Adminhtml\Import\Edit\Renderer\Uploader::class
            )
        );

        $form->setUseContainer(true);
        $this->setForm($form);

        return parent::_prepareForm();
    }

    /**
     * Get download sample file html
     *
     * @return string
     */
    protected function getDownloadSampleFileHtml()
    {
        $html = '<span id="sample-file-span" class="no-display"><a id="sample-file-link" href="#">'
                . __('Download Sample File')
                . '</a></span>';
        return $html;
    }

    protected function getCsvFileInfoToolTip()
    {
        return $this->getToolTipHtml(
            __('File format should be CSV(comma-separated-value) with a comma (,) as a delimiter, double quotes(") as enclosure and UTF-8 as data encoding.')
            . '<br /><br /><strong>' . __('Note') . ': </strong>'
            . __('We recommend that you use a tool that supports UTF-8 encoding, such as <a href="https://docs.google.com/spreadsheets">Google Sheets</a> or <a href="https://www.openoffice.org/" target="_blank">OpenOffice Calc</a>, to edit CSV files')
        );
    }

    protected function getBehaviorInfoToolTip()
    {
        return $this->getToolTipHtml(
            '<strong>' . __('Add/Update') . ':</strong> '
            . __('New data is added and existing data is updated.')
            . '<br /><strong>' . __('Replace') . ':</strong> '
            . __('The existing data is replaced with new data.')
            . '<br /><strong>' . __('Delete') . ':</strong> '
            . __('The existing data is deleted.')
            . '<br /><span style="font-size: 18px;color: #D83900;position: relative;top: 2px;">⚠</span>'
            . '<em>' . __('use with caution'). '</em>'
        );
    }

    protected function getValidationStrategyInfoToolTip()
    {
        return $this->getToolTipHtml(
            __('Determine what happens when an error is encountered when importing data.')
        );
    }

    protected function getAllowedErrorsCountInfoToolTip()
    {
        return $this->getToolTipHtml(
            __('Number of errors to halt import process.')
        );
    }

    protected function getToolTipHtml($text)
    {
        return '<div class="admin__field-tooltip">
        <a class="admin__field-tooltip-action action-help" tabindex="1" onclick="return false;"></a>
    <div class="admin__field-tooltip-content">
        ' . $text . '
    </div>
</div>';
    }
}
