<?php

namespace MagePsycho\MipPricing\Model\Import\Pricing\Validator;

use MagePsycho\MipPricing\Model\Import\Pricing\RowValidatorInterface;
use MagePsycho\MipPricing\Model\Import\Pricing as ImportPricing;
use Magento\Framework\Stdlib\DateTime;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class DateFromTo extends AbstractImportValidator implements RowValidatorInterface
{
    /**
     * @var DateTime
     */
    private $dateTime;

    public function __construct(
        DateTime $dateTime
    ) {
        $this->dateTime = $dateTime;
    }

    /**
     * {@inheritdoc}
     */
    public function isValid($value)
    {
        $this->_clearMessages();

        if (isset($value[ImportPricing::COL_SPECIAL_FROM_DATE])
            && strlen($value[ImportPricing::COL_SPECIAL_FROM_DATE])
        ) {
            if (strtotime($value[ImportPricing::COL_SPECIAL_FROM_DATE]) === false) {
                $this->_addMessages([self::ERROR_INVALID_DATE]);
                return false;
            }
        }

        if (isset($value[ImportPricing::COL_SPECIAL_TO_DATE])
            && strlen($value[ImportPricing::COL_SPECIAL_TO_DATE])
        ) {
            if (strtotime($value[ImportPricing::COL_SPECIAL_TO_DATE]) === false) {
                $this->_addMessages([self::ERROR_INVALID_DATE]);
                return false;
            }
        }

        if (!empty($value[ImportPricing::COL_SPECIAL_FROM_DATE])
            && !empty($value[ImportPricing::COL_SPECIAL_TO_DATE])
        ) {
            $specialFromTimestamp = strtotime(
                $this->dateTime->formatDate($value[ImportPricing::COL_SPECIAL_FROM_DATE], false)
            );
            $specialToTimestamp   = strtotime(
                $this->dateTime->formatDate($value[ImportPricing::COL_SPECIAL_TO_DATE], false)
            );
            if ($specialFromTimestamp > $specialToTimestamp) {
                $this->_addMessages([self::ERROR_INVALID_TO_DATE]);
                return false;
            }
        }

        return true;
    }
}
