<?php

namespace MagePsycho\MipPricing\Model\Import\Pricing\Validator;

use MagePsycho\MipPricing\Model\Import\Pricing\RowValidatorInterface;
use MagePsycho\MipPricing\Model\Import\Pricing as ImportPricing;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class DecimalPrice extends AbstractImportValidator implements RowValidatorInterface
{
    /**
     * {@inheritdoc}
     */
    public function isValid($value)
    {
        $this->_clearMessages();

        if (isset($value[ImportPricing::COL_PRICE]) && strlen($value[ImportPricing::COL_PRICE])) {
            if (!$this->assertMainPriceFormat($value[ImportPricing::COL_PRICE])) {
                $this->_addMessages([self::ERROR_INVALID_PRICE_FORMAT]);
                return false;
            }
        }

        if (isset($value[ImportPricing::COL_COST]) && strlen($value[ImportPricing::COL_COST])) {
            if (!$this->assertMainPriceFormat($value[ImportPricing::COL_COST])
                && strtolower($value[ImportPricing::COL_COST]) != self::DELETE_MARKER
            ) {
                $this->_addMessages([self::ERROR_INVALID_PRICE_FORMAT]);
                return false;
            }
        }

        if (isset($value[ImportPricing::COL_MSRP]) && strlen($value[ImportPricing::COL_MSRP])) {
            if (!$this->assertPriceFormat($value[ImportPricing::COL_MSRP])) {
                $this->_addMessages([self::ERROR_INVALID_PRICE_FORMAT]);
                return false;
            }
        }

        if (isset($value[ImportPricing::COL_SPECIAL_PRICE]) && strlen($value[ImportPricing::COL_SPECIAL_PRICE])) {
            if (!$this->assertPriceFormat($value[ImportPricing::COL_SPECIAL_PRICE])) {
                $this->_addMessages([self::ERROR_INVALID_PRICE_FORMAT]);
                return false;
            }
        }

        return true;
    }

    /**
     * Supports different price formats:
     * x
     * X
     * +10%
     * -10%
     * +10
     * -10
     * 10%
     * 10
     * 10.50
     * etc.
     *
     * @param $value
     * @return false|int
     */
    public function assertPriceFormat($value)
    {
        if (strtolower($value) == self::DELETE_MARKER) {
            return true;
        }
        return preg_match('/^[+-]?(\d+\.)?(\d+)[%]?$/', $value);
    }

    /**
     * @see assertPriceFormat()
     * Same as above except format: x, X & N%
     *
     * @param $value
     * @return false|int
     */
    public function assertMainPriceFormat($value)
    {
        return preg_match('/^([+-])?\d+(?:\.\d+)?(?(1)%)?$/', $value);
    }
}
