<?php

namespace MagePsycho\MipPricing\Model\Import\Pricing\Validator;

use MagePsycho\MipPricing\Model\Import\Pricing\RowValidatorInterface;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class TierPrice extends AbstractPrice implements RowValidatorInterface
{
    const PRICE_PREFIX           = 'tier_price';
    const QTY_TO_PRICE_SEPARATOR = '=';
    const MULTI_VALUES_SEPARATOR = ';';

    /**
     * @var \MagePsycho\MipPricing\Model\Import\Pricing\StoreResolver
     */
    protected $storeResolver;

    public function __construct(
        \Magento\Customer\Api\GroupRepositoryInterface $groupRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \MagePsycho\MipPricing\Model\Import\Pricing\StoreResolver $storeResolver
    ) {
        $this->storeResolver = $storeResolver;
        parent::__construct($groupRepository, $searchCriteriaBuilder);
    }

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    public function isValid($value)
    {
        $this->_clearMessages();

        $tierPriceGroups = array_merge([self::VALUE_ALL], $this->customerGroups);
        foreach ($tierPriceGroups as $priceGroup) {
            if (isset($value[self::PRICE_PREFIX . ':' . $priceGroup])) {
                $tierPrice = $value[self::PRICE_PREFIX . ':' . $priceGroup];
                if (!strlen($tierPrice)) {
                    continue;
                }

                if (strtolower($tierPrice) == self::DELETE_MARKER) {
                    continue;
                }

                $tierQtyToPrices = explode(
                    self::MULTI_VALUES_SEPARATOR,
                    trim($tierPrice, self::MULTI_VALUES_SEPARATOR)
                );

                foreach ($tierQtyToPrices as $tierQtyToPrice) {
                    list($tierQty, $tierPrice) = explode(self::QTY_TO_PRICE_SEPARATOR, $tierQtyToPrice);
                    if ((!strlen($tierQty) || !is_numeric($tierQty))
                        || (!strlen($tierPrice) || !$this->assertPriceFormat($tierPrice))
                    ) {
                        $this->_addMessages([self::ERROR_TIER_DATA_INVALID]);
                        return false;
                    }
                }
            }
        }

        return true;
    }
}
