<?php

namespace MagePsycho\MipPricing\Model\Import\Pricing\Validator;

use MagePsycho\MipPricing\Model\Import\Pricing\RowValidatorInterface;
use MagePsycho\MipPricing\Model\Import\Pricing as ImportPricing;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Website extends AbstractImportValidator implements RowValidatorInterface
{
    const DEFAULT_WEBSITE_ID = 0;

    /**
     * @var ImportPricing\StoreResolver
     */
    protected $storeResolver;

    public function __construct(
        ImportPricing\StoreResolver $storeResolver
    ) {
        $this->storeResolver = $storeResolver;
    }

    /**
     * {@inheritdoc}
     */
    public function isValid($value)
    {
        $this->_clearMessages();
        if (!isset($value[ImportPricing::COL_WEBSITE_ID])) {
            return false;
        }

        $websiteId = $value[ImportPricing::COL_WEBSITE_ID];
        if ($value[ImportPricing::COL_WEBSITE_ID] != self::DEFAULT_WEBSITE_ID
            && !$this->storeResolver->checkWebsiteId($websiteId)
        ) {
            $this->_addMessages([self::ERROR_INVALID_WEBSITE]);
            return false;
        }

        return true;
    }
}
