<?php

namespace MagePsycho\MipPricing\Model\System\Config\Source;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class PriceAttribute implements \Magento\Framework\Option\ArrayInterface
{
    const ATTR_NAME              = 'name';
    const ATTR_COST              = 'cost';
    const ATTR_PRICE             = 'price';
    const ATTR_MSRP              = 'msrp';
    const ATTR_SPECIAL_PRICE     = 'special_price';
    const ATTR_TIER_PRICE        = 'tier_price';

    protected $_options;

    public function getAllOptions($withEmpty = false)
    {
        if ($this->_options === null) {
            $this->_options = [
                [
                    'value' => self::ATTR_NAME,
                    'label' => __('Name'),
                ],
                [
                    'value' => self::ATTR_COST,
                    'label' => __('Cost'),
                ],
                [
                    'value' => self::ATTR_PRICE,
                    'label' => __('Price'),
                ],
                [
                    'value' => self::ATTR_MSRP,
                    'label' => __('MSRP'),
                ],
                [
                    'value' => self::ATTR_SPECIAL_PRICE,
                    'label' => __('Special Price'),
                ],
                [
                    'value' => self::ATTR_TIER_PRICE,
                    'label' => __('Tier Price'),
                ],
            ];

        }
        $options = $this->_options;
        if ($withEmpty) {
            array_unshift($options, ['value' => '', 'label' => '']);
        }
        return $options;
    }

    public function getOptionsArray($withEmpty = true)
    {
        $options = [];
        foreach ($this->getAllOptions($withEmpty) as $option) {
            $options[$option['value']] = $option['label'];
        }
        return $options;
    }

    public function getOptionText($value)
    {
        $options = $this->getAllOptions(false);
        foreach ($options as $item) {
            if ($item['value'] == $value) {
                return $item['label'];
            }
        }
        return false;
    }

    public function toOptionArray()
    {
        return $this->getAllOptions();
    }

    public function toOptionHash($withEmpty = true)
    {
        return $this->getOptionsArray($withEmpty);
    }
}
