<?php

namespace MagePsycho\MipPricing\Plugin\Block\Export;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Form
{
    /**
     * @var \MagePsycho\MipPricing\Model\System\Config\Source\PriceAttribute
     */
    private $priceAttributeConfig;

    /**
     * @var \MagePsycho\MipPricing\Model\System\Config\Source\Website
     */
    private $websiteOptionHash;

    public function __construct(
        \MagePsycho\MipPricing\Model\System\Config\Source\PriceAttribute $priceAttributeConfig,
        \MagePsycho\MipPricing\Model\System\Config\Source\Website $websiteOptionHash
    ) {
        $this->priceAttributeConfig = $priceAttributeConfig;
        $this->websiteOptionHash = $websiteOptionHash;
    }

    public function beforeSetForm(
        \Magento\ImportExport\Block\Adminhtml\Export\Edit\Form $subject,
        $form
    ) {
        $fieldset = $form->getElement('base_fieldset');
        $priceAttributeOptions = $this->priceAttributeConfig->getAllOptions();
        $websiteOptions = $this->websiteOptionHash->toOptionArray();
        $fieldset->addField(
            'mip_website_id',
            'select',
            [
                'name'      => 'mip_website_id',
                'label'     => __('Website'),
                'title'     => __('Website'),
                'required'  => true,
                'class'     => 'input-text',
                'values'    => $websiteOptions,
                'value'     => ''
            ]
        );
        $fieldset->addField(
            'mip_pricing_attributes',
            'multiselect',
            [
                'name'      => 'mip_pricing_attributes',
                'label'     => __('MIP Price Attributes'),
                'title'     => __('MIP Price Attributes'),
                'required'  => true,
                'class'     => 'input-text',
                'values'    => $priceAttributeOptions,
                'value'     => implode(',', array_column($priceAttributeOptions, 'value'))
            ]
        )->setAfterElementHtml('
        <div class="admin__field-tooltip">
        <a class="admin__field-tooltip-action action-help" tabindex="1" onclick="return false;"></a>
    <div class="admin__field-tooltip-content">Select the price attributes to export.<br/><strong>Note:</strong> <em>Name</em> field will be only used as a reference.</div>
</div>
    ');
        return [$form];
    }
}
