<?php
/**
 * Copyright © MageWorx. All rights reserved.
 * See LICENSE.txt for license details.
 */

namespace MageWorx\ShippingRules\Ui\Component\Listing\Columns;

use MageWorx\ShippingRules\Controller\Adminhtml\Shippingrules\Rate as RateController;
use Magento\Ui\Component\Listing\Columns\Column;

/**
 * Class RateActions
 */
class RateActions extends Column
{
    /**
     * Url path  to edit
     *
     * @var string
     */
    const URL_PATH_EDIT = 'mageworx_shippingrules/shippingrules_rate/edit';

    /**
     * Url path  to delete
     *
     * @var string
     */
    const URL_PATH_DELETE = 'mageworx_shippingrules/shippingrules_rate/delete';

    /**
     * URL builder
     *
     * @var \Magento\Framework\UrlInterface
     */
    protected $urlBuilder;

    /**
     * Constructor
     *
     * @param \Magento\Framework\UrlInterface $urlBuilder
     * @param \Magento\Framework\View\Element\UiComponent\ContextInterface $context
     * @param \Magento\Framework\View\Element\UiComponentFactory $uiComponentFactory
     * @param array $components
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Framework\View\Element\UiComponent\ContextInterface $context,
        \Magento\Framework\View\Element\UiComponentFactory $uiComponentFactory,
        array $components = [],
        array $data = []
    ) {
        $this->urlBuilder = $urlBuilder;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }

    /**
     * Prepare Data Source
     *
     * @param array $dataSource
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        if (!isset($dataSource['data']['items'])) {
            return $dataSource;
        }

        foreach ($dataSource['data']['items'] as &$item) {
            if (!isset($item['rate_id'])) {
                continue;
            }

            $editParams = [];
            if ($this->context->getNamespace() === 'mageworx_shippingrules_rates_listing') {
                $editParams[RateController::BACK_TO_PARAM] = RateController::BACK_TO_METHOD_PARAM;
            }
            $editParams['id']             = $item['rate_id'];
            $item[$this->getData('name')] = [
                'edit'   => [
                    'href'  => $this->urlBuilder->getUrl(
                        static::URL_PATH_EDIT,
                        $editParams
                    ),
                    'label' => __('Edit'),
                ],
                'delete' => [
                    'href'    => $this->urlBuilder->getUrl(
                        static::URL_PATH_DELETE,
                        $editParams
                    ),
                    'label'   => __('Delete'),
                    'confirm' => [
                        'title'   => __('Delete "${ $.$data.title }"'),
                        'message' => __('Are you sure you wan\'t to delete the rate "${ $.$data.title }" ?'),
                        '__disableTmpl' => [
                            'title' => false,
                            'message' => false
                        ]
                    ],
                ],
            ];
        }

        return $dataSource;
    }
}
