<?php
namespace Magecomp\Matrixrate\Model\Carrier;

use Magento\Framework\Exception\LocalizedException;
use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Shipping\Model\Carrier\AbstractCarrier;
use Magento\Shipping\Model\Carrier\CarrierInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory;
use Psr\Log\LoggerInterface;
use Magento\Shipping\Model\Rate\ResultFactory;
use Magento\Quote\Model\Quote\Address\RateResult\MethodFactory;
use Magecomp\Matrixrate\Model\ResourceModel\Carrier\MatrixrateFactory;
use Magento\Framework\App\ObjectManager;

class Matrixrate extends AbstractCarrier implements CarrierInterface
{

    protected $_code = 'matrixrate';

    protected $_isFixed = true;

    protected $_defaultConditionName = 'package_weight';

    protected $_conditionNames = [];
    protected $_cartOrder;

    /**
     * @var \Magento\Shipping\Model\Rate\ResultFactory
     */
    public $_rateResultFactory;

    /**
     * @var \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory
     */
    protected $_rateMethodFactory;

    /**
     * @var \Ced\Advancedmatrix\Model\Resource\Carrier\AdvancedrateFactory
     */
    protected $_tablerateFactory;


    public function __construct(
        ScopeConfigInterface $scopeConfig,
        ErrorFactory $rateErrorFactory,
        LoggerInterface $logger,
        ResultFactory $rateResultFactory,
        MethodFactory $rateMethodFactory,
        MatrixrateFactory $tablerateFactory,
        \Magento\Checkout\Model\Cart $_cartOrder,
        array $data = []
    ) {
    	$this->_scopeConfig = $scopeConfig;
        $this->_rateResultFactory = $rateResultFactory;
        $this->_rateMethodFactory = $rateMethodFactory;
        $this->_tablerateFactory = $tablerateFactory;
        $this->_cartOrder = $_cartOrder;
        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
		foreach ($this->getCode('condition_name') as $k => $v) {
            $this->_conditionNames[] = $k;
        }

     }

    /**
     * @param RateRequest $request
     * @return \Magento\Shipping\Model\Rate\Result
     */

   
    public function collectRates(RateRequest $request)
    {
           
       if (!$this->getConfigFlag('active')) {
           return false;
        }
     
        $freeQty = 0;

        if (!$request->getConditionMRName()) {
            $conditionName = $this->getConfigData('condition_name');
            $request->setConditionMRName($conditionName ? $conditionName : $this->_defaultConditionName);
        }

        // Package weight and qty free shipping
        $oldWeight = $request->getPackageWeight();
        $oldQty = $request->getPackageQty();
		
        /** @var \Magento\Shipping\Model\Rate\Result $result */
        $result = $this->_rateResultFactory->create();
        //$zipRange = $this->getConfigData('zip_range');
        $rateArray = $this->getRate($request);
        $request->setPackageWeight($oldWeight);
        $request->setPackageQty($oldQty);

        $foundRates = false;
		$freeShipping=false;
     	
        foreach ($rateArray as $rate)
		{
            if (!empty($rate) && $rate['price'] >= 0)
			{

                /** @var \Magento\Quote\Model\Quote\Address\RateResult\Method $method */
                $method = $this->_rateMethodFactory->create();

                $method->setCarrier('matrixrate');
                $method->setCarrierTitle($this->getConfigData('title'));

                $method->setMethod('matrixrate_' . $rate['id']);
                $method->setMethodTitle(__($rate['delivery_type']));

                if ($request->getPackageQty() == $freeQty) {
                    $shippingPrice = 0;
                } else {
                    $shippingPrice = $this->getFinalPriceWithHandlingFee($rate['price']);
                }
                $method->setPrice($shippingPrice);
                $method->setCost($rate['cost']);

                $result->append($method);
                $foundRates = true; // have found some valid rates
            }
        }

        if (!$foundRates){
            /** @var \Magento\Quote\Model\Quote\Address\RateResult\Error $error */
            $error = $this->_rateErrorFactory->create(
                [
                    'data' => [
                        'carrier' => $this->_code,
                        'carrier_title' => $this->getConfigData('title'),
                        'error_message' => $this->getConfigData('specificerrmsg'),
                    ],
                ]
            );
            $result->append($error);
        }
        return $result;
                
    }

    public function getRate(RateRequest $request)
    {
        return $this->_tablerateFactory->create()->getNewRate($request);
    }

    public function getAllowedMethods()
    {
      return ['matrixrate'=> $this->getConfigData('name')];
    }
	
	public function getCode($type, $code='')
	{
        $codes = array(

            'condition_name'=>array(
                'package_weight' => __('Weight vs. Destination'),
                'package_value'  => __('Price vs. Destination'),
               // 'package_qty'    => __('# of Items vs. Destination'),
            ),

            'condition_name_short'=>array(
                'package_weight' => __('Weight'),
                'package_value'  => __('Order Subtotal'),
                //'package_qty'    => __('# of Items'),
            ),

        );

        if (!isset($codes[$type])) {
            throw new Magento\Framework\Exception\LocalizedException__('Invalid Matrix Rate code type: %1', $type);
        }

        if (''===$code) {
            return $codes[$type];
        }

        if (!isset($codes[$type][$code])) {
            throw new Magento\Framework\Exception\LocalizedException__('Invalid Matrix Rate code for type %1: %2', $type, $code);
        }

        return $codes[$type][$code];
    }

}
