<?php
namespace Magecomp\Matrixrate\Model\ResourceModel\Carrier;

use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Framework\DB\Select;
use Magento\Framework\App\ObjectManager;

class RateQuery
{
    /**
     * @var \Magento\Quote\Model\Quote\Address\RateRequest
     */
    private $request;
	
	protected $_mysql;

    /**
     * RateQuery constructor.
     * @param \Magento\Quote\Model\Quote\Address\RateRequest $request
     */
    public function __construct(
        RateRequest $request
	) {
        $this->request = $request;
	}

    /**
     * @param \Magento\Framework\DB\Select $select
     * @return \Magento\Framework\DB\Select
     */
    public function prepareSelect(Select $select)
    {

		$postcode = $this->request->getDestPostcode();
       
        $zipRangeSet = 1;
		if ($zipRangeSet || is_numeric($postcode)) {
             $zipSearchString = '
              AND
                 IF(dest_zip="*",
                   IF(dest_zip_to="*",true,:postcode <= CAST(dest_zip_to AS UNSIGNED)) ,
                   IF(dest_zip_to="*",:postcode >= CAST(dest_zip AS UNSIGNED), :postcode BETWEEN CAST(dest_zip AS UNSIGNED) AND  CAST(dest_zip_to AS UNSIGNED)))
              ';
		} else {
			$zipSearchString = ' AND :postcode LIKE dest_zip';
		}

        $orWhere = '(' . implode(
            ') OR (',
            [
                "IF(dest_country_id  NOT IN(:country_id), dest_country_id='0', dest_country_id=:country_id) AND IF(dest_region_id  <> :region_id , dest_region_id=0, dest_region_id=:region_id)  AND IF(dest_city = :city , dest_city=:city , dest_city='*') ". $zipSearchString ,
            ]
        ) . ')';

                                
        $select->where($orWhere);

        // Render condition by condition name
        if (is_array($this->request->getConditionMRName()))
        {
            $orWhere = [];
            foreach (range(0, count($this->request->getConditionMRName())) as $conditionNumber) {
                $bindNameKey = sprintf(':condition_name_%d', $conditionNumber);
                $bindValueKey = sprintf(':condition_from_value_%d', $conditionNumber);
                $bindToKey = sprintf(':condition_to_value_%d', $conditionNumber);
                $orWhere[] = "(condition_name = {$bindNameKey} AND condition_value <= {$bindValueKey} AND condition_value >= {$bindToKey})";
            }

            if ($orWhere) {
                $select->where(implode(' OR ', $orWhere));
            }
        } else {
                            
            $select->where('condition_name = :condition_name');
            $select->where('condition_from_value <= :condition_value');
            $select->where('condition_to_value >= :condition_value');
        }
        
        $select->where('website_id=?', $this->request->getWebsiteId());
           
		return $select;
    }

    /**
     * @return array
     */
    public function getBindings($select)
    {
       
        $bind = [];
        $bind = [
            ':website_id' => (int)$this->request->getWebsiteId(),
            ':country_id' => ( $this->request->getDestCountryId() == "null" || $this->request->getDestCountryId() == 0) ? "0" : $this->request->getDestCountryId(),
            ':region_id' => (int)$this->request->getDestRegionId(),
            ':postcode' => ( $this->request->getDestPostcode() == "null" || $this->request->getDestPostcode() == "") ? "0" : $this->request->getDestPostcode(),
			':city' => ($this->request->getDestCity() == "null" || $this->request->getDestCity() == "") ? "*" : $this->request->getDestCity(),
        ];

               
        // Render condition by condition name
        if (is_array($this->request->getConditionMRName())) {
            $i = 0;
            foreach ($this->request->getConditionMRName() as $conditionName) {
                $bindNameKey = sprintf(':condition_name_%d', $i);
                $bindValueKey = sprintf(':condition_value_%d', $i);
                $bind[$bindNameKey] = $conditionName;
                $bind[$bindValueKey] = $this->request->getData($conditionName);
                $i++;
            }
        } else {
            $bind[':condition_name'] = $this->request->getConditionMRName();
            $bind[':condition_value'] = $this->request->getData($this->request->getConditionMRName());
        }
	
        ObjectManager::getInstance()->get('Psr\Log\LoggerInterface')->debug(print_r(['Get Bindings' => $bind], true));
        return $bind;
    }

    /**
     * @return \Magento\Quote\Model\Quote\Address\RateRequest
     */
    public function getRequest()    
    {
        return $this->request;
    }
}
