<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

namespace Magefan\GuestToCustomer\Model;

use Magento\Store\Model\ScopeInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;


/**
 * Class Config
 *
 */
class Config
{
    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_ENABLED = 'mfguesttocustomer/general/enabled';

    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_AUTOCONVERT_ENABLED = 'mfguesttocustomer/general/autoconvert_order';

    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_GROUP_ID = 'mfguesttocustomer/general/customer_group';

    /**
     * @var string
     */
    const XML_PATH_GUEST_TO_CUSTOMER_ASSIGN_ADDRESS = 'mfguesttocustomer/general/assign_address';

    /**
     * @var string
     */
    const XML_PATH_NOTIFICATION_MESSAGE_ENABLED = 'mfguesttocustomer/general/send_email';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * Config constructor.
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param null $storeId
     * @return mixed
     */
    public function isEnabled($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_ENABLED,
            $storeId
        );
    }

    /**
     * @param null $storeId
     * @return mixed
     */
    public function isAutoconvertEnabled($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_AUTOCONVERT_ENABLED,
            $storeId
        );
    }

    /**
     * @param $path
     * @param $storeId
     * @return mixed
     */
    public function getConfig($path, $storeId)
    {
        return $this->scopeConfig->getValue(
            $path,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @param null $storeId
     * @return mixed
     */
    public function getCustomerGroupId($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_GROUP_ID,
            $storeId
        );
    }

    /**
     * @param null $storedId
     * @return mixed
     */
    public function isAssignOrderAddressToCustomer($storedId = null)
    {
        return $this->getConfig(
            self::XML_PATH_GUEST_TO_CUSTOMER_ASSIGN_ADDRESS,
            $storedId
        );
    }

    /**
     * @param null $storedId
     * @return mixed
     */
    public function isNotificationMessagesEnabled($storeId = null)
    {
        return $this->getConfig(
            self::XML_PATH_NOTIFICATION_MESSAGE_ENABLED,
            $storeId
        );
    }

}
