<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

namespace Magefan\GuestToCustomer\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;
use Psr\Log\LoggerInterface;
use Magefan\GuestToCustomer\Model\Config;
use Magefan\GuestToCustomer\Model\Order\CustomerManagement;
use Magento\Framework\Indexer\IndexerRegistry;

/**
 * Class SalesOrderSaveAfter save order with new customer.
 */
class SalesOrderSaveAfter implements ObserverInterface
{
    /**
     * @var Config
     */
    protected $config;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var IndexerRegistry
     */
    protected $indexerRegistry;

    /**
     * SalesOrderSaveAfter constructor.
     * @param Config $config
     * @param LoggerInterface $logger
     * @param CustomerManagement $customerManagement
     * @param IndexerRegistry $indexerRegistry
     */
    public function __construct(
        Config $config,
        LoggerInterface $logger,
        CustomerManagement $customerManagement,
        IndexerRegistry $indexerRegistry = null
    ) {
        $this->config = $config;
        $this->logger = $logger;
        $this->customerManagement = $customerManagement;
        $this->indexerRegistry = $indexerRegistry ?: \Magento\Framework\App\ObjectManager::getInstance()
            ->get(\Magento\Framework\Indexer\IndexerRegistry::class);
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(Observer $observer)
    {
        $order = $observer->getEvent()->getOrder();
        if (!$order || !$order->getId() || $order->getCustomerId()) {
            return;
        }

        try {
            $storeId = $order->getStoreId();
            if ($this->config->isEnabled($storeId)
                && $this->config->isAutoconvertEnabled($storeId)) {
                /*convert guest to customer*/
                $this->customerManagement->convertGuestOrderById($order->getId());
                /*reindex new customer */
                if ($order->getCustomerId()) {
                    // Get index for each action separately
                    $this->indexerRegistry->get(\Magento\Customer\Model\Customer::CUSTOMER_GRID_INDEXER_ID)->invalidate();
                    $this->indexerRegistry->get(\Magento\Customer\Model\Customer::CUSTOMER_GRID_INDEXER_ID)->reindexRow($order->getCustomerId());
                }
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        }
    }
}
