<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */
namespace Magefan\GuestToCustomer\Plugin\Magento\Sales\Model\Order;

use Magento\Sales\Api\OrderRepositoryInterface;

/**
 * Class OrderCustomerExtractor get customer from Order.
 */
class OrderCustomerExtractor
{
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @param OrderRepositoryInterface $orderRepository
     */
    public function __construct(
        OrderRepositoryInterface $orderRepository
    ) {
        $this->orderRepository = $orderRepository;
    }

    /**
     * @param $subject
     * @param $customer
     */
    public function afterExtract(
        $subject,
        $customer,
        $orderId
    ) {
        $order = $this->orderRepository->get($orderId);

        $store = $order->getStore();
        if ($store && $store->getWebsiteId()) {
            $customer->setWebsiteId($store->getWebsiteId());
        }
        $customer->setStoreId($order->getStoreId());

        if (!$customer->getTaxvat()) {
            $vatFields = ['vat_id'];
            foreach ($vatFields as $vatField) {
                $taxvatValue = $order->getBillingAddress()->getData($vatField);
                if ($taxvatValue) {
                    $customer->setTaxvat($taxvatValue);
                    break;
                }
            }
        }

        return $customer;
    }
}
