<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_LayeredNavigation
 * @copyright   Copyright (c) 2017 Mageplaza (http://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\LayeredNavigation\Model\Search;

use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\ObjectFactory;
use Magento\Framework\Api\Search\FilterGroup;
use Magento\Framework\Api\Search\FilterGroupBuilder as SourceFilterGroupBuilder;
use Magento\Framework\App\RequestInterface;

/**
 * Builder for FilterGroup Data.
 */
class FilterGroupBuilder extends SourceFilterGroupBuilder
{
	/** @var \Magento\Framework\App\RequestInterface */
	protected $_request;

	/**
	 * FilterGroupBuilder constructor.
	 * @param \Magento\Framework\Api\ObjectFactory $objectFactory
	 * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
	 * @param \Magento\Framework\App\RequestInterface $request
	 */
	public function __construct(
		ObjectFactory $objectFactory,
		FilterBuilder $filterBuilder,
		RequestInterface $request
	)
	{
		parent::__construct($objectFactory, $filterBuilder);

		$this->_request = $request;
	}

	/**
	 * @return FilterGroupBuilder
	 */
	public function cloneObject()
	{
		$cloneObject = clone $this;
		$cloneObject->setFilterBuilder(clone $this->_filterBuilder);

		return $cloneObject;
	}

	/**
	 * @param $filterBuilder
	 */
	public function setFilterBuilder($filterBuilder)
	{
		$this->_filterBuilder = $filterBuilder;
	}

	/**
	 * @param $attributeCode
	 *
	 * @return $this
	 */
	public function removeFilter($attributeCode)
	{
		if (isset($this->data[FilterGroup::FILTERS])) {
			foreach ($this->data[FilterGroup::FILTERS] as $key => $filter) {
				if ($filter->getField() == $attributeCode) {
					if (($attributeCode == 'category_ids') && ($filter->getValue() == $this->_request->getParam('id'))) {
						continue;
					}
					unset($this->data[FilterGroup::FILTERS][$key]);
				}
			}
		}

		return $this;
	}

	/**
	 * Return the Data type class name
	 *
	 * @return string
	 */
	protected function _getDataObjectType()
	{
		return 'Magento\Framework\Api\Search\FilterGroup';
	}
}
