<?php

namespace Nexus\Catalog\Controller\Adminhtml\Categoryimport;

class ImportLists extends \Magento\Backend\App\Action
{
    protected $_backendSession;
 
    protected $_storeManager;

    public function __construct(
        \Magento\Backend\Model\Session $backendSession,      
        \Magento\Framework\Registry $registry,
        \Magento\Backend\Model\View\Result\RedirectFactory $resultRedirectFactory,
        \Magento\Backend\App\Action\Context $context,       
		\Magento\Framework\Filesystem $filesystem,       
		\Magento\MediaStorage\Model\File\UploaderFactory $fileUploaderFactory,
        \Nexus\Catalog\Model\CategoryPositionFactory $categoryPositionFactory,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Framework\File\Csv $csv
    )
    {
        $this->_backendSession = $backendSession;
        $this->context = $context;      
        $this->_filesystem = $filesystem;       
		$this->_mediaDirectory = $filesystem->getDirectoryWrite(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA);
		$this->_fileUploaderFactory = $fileUploaderFactory;		
        $this->csv = $csv;
        $this->_categoryPosition = $categoryPositionFactory;
        $this->productRepository = $productRepository;
        parent::__construct($context);
    }

    /**
     * run the action
     *
     * @return \Magento\Backend\Model\View\Result\Redirect
     */
    public function execute()
    {
        $data = $this->getRequest()->getFiles();
        $filename = $data['file']['name'];
        $extension = pathinfo($filename, PATHINFO_EXTENSION);
        
        $resultRedirect = $this->resultRedirectFactory->create();
                  
        if (!isset($data['file']['tmp_name'])){ 
      
            $this->messageManager->addError(__("Invalid file upload attempt."));
            $resultRedirect->setPath('*/categoryimport/import');
            return $resultRedirect;        
        }
        
        try {        

            $csvData = $this->csv->getData($data['file']['tmp_name']);
            $header= $csvData[0];
            array_shift($csvData);
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
            $connection = $resource->getConnection();
            $connection->query("SET FOREIGN_KEY_CHECKS=0;");           
            $itemsPosition = 0;
            $itemsDeleted = 0;
            $newcsvrecords = 0;      
                 
            foreach ($csvData as $key => $value) {
               /* if(empty($value[0]) || empty($value[2]) || empty($value[1]) || empty($value[3])){
                    $row = $key;
                    $this->messageManager->addError(__("Please add values in Row ".$row));
                    continue;
                }*/
                   $cp = $this->_categoryPosition->create();
                   $categoryid =  $value[0];
                   $product_id = $this->productRepository->get($value[1])->getId();
                   $position =  $value[2];
                   $action =  $value[3];
                   $collection =  $cp->getCollection()->addFieldToFilter('category_id',$categoryid)->addFieldToFilter('product_id',$product_id);
                    if($collection->getSize()){
                        foreach ($collection as $item) {
                            if($action){                               
                               $item->setData('position',$position)->save(); 
                               $itemsPosition++; 
                            }else{
                                $item->delete();
                                $itemsDeleted++;
                            }
                            
                        }
                       

                    }else{
                        if($action){
                            $cp->setData('category_id',$categoryid);
                            $cp->setData('product_id',$product_id);
                            $cp->setData('position',$position);
                            $cp->save();
                            $newcsvrecords++;  
                        } 


                    }   
               

            }      
         
            
			$connection->query("SET FOREIGN_KEY_CHECKS=1;");
            $this->messageManager->addSuccess(__("Import has been successfully finished."));
            if($itemsPosition){
                $this->messageManager->addSuccess(__("Total $itemsPosition products position has been updated."));
            }
            if($itemsDeleted){
                $this->messageManager->addSuccess(__("Total $itemsDeleted products  has been deleted."));
            }
             if($newcsvrecords){
                $this->messageManager->addSuccess(__("Total $newcsvrecords products  has been linked to categories."));
            }
			
            $resultRedirect->setPath('*/categoryimport/import');
            
			return $resultRedirect;

		} catch (\Exception $e) {
			$this->messageManager->addError(__($e->getMessage()));
			$resultRedirect->setPath('*/categoryimport/import');
			return $resultRedirect;
		}
		
    }

}
